@@ -1,5 +1,6 @@
 #! /usr/bin/env python
 
+import argparse
 import base64
 import hashlib
 import hmac
@@ -104,42 +105,61 @@ class CloudStack(object):
             return self._request(command, **kwargs)
         return handler
 
-    def _request(self, command, json=True, opcode_name='command', **kwargs):
+    def _request(self, command, json=True, opcode_name='command',
+                 fetch_list=False, **kwargs):
         kwargs.update({
             'apiKey': self.key,
             opcode_name: command,
         })
         if json:
             kwargs['response'] = 'json'
-        if 'page' in kwargs:
+        if 'page' in kwargs or fetch_list:
             kwargs.setdefault('pagesize', 500)
 
-        kwargs = transform(kwargs)
-        kwargs['signature'] = self._sign(kwargs)
+        kwarg = 'params' if self.method == 'get' else 'data'
 
-        kw = {'timeout': self.timeout}
-        if self.method == 'get':
-            kw['params'] = kwargs
-        else:
-            kw['data'] = kwargs
-        response = getattr(requests, self.method)(self.endpoint, **kw)
+        done = False
+        final_data = []
+        page = 1
+        while not done:
+            if fetch_list:
+                kwargs['page'] = page
 
-        try:
-            data = response.json()
-        except ValueError as e:
-            msg = "Make sure endpoint URL '%s' is correct." % self.endpoint
-            raise CloudStackException(
-                "HTTP {0} response from CloudStack".format(
-                    response.status_code), response, "%s. " % str(e) + msg
-                )
+            kwargs = transform(kwargs)
+            kwargs.pop('signature', None)
+            kwargs['signature'] = self._sign(kwargs)
 
-        [key] = data.keys()
-        data = data[key]
-        if response.status_code != 200:
-            raise CloudStackException(
-                "HTTP {0} response from CloudStack".format(
-                    response.status_code), response, data)
-        return data
+            response = getattr(requests, self.method)(self.endpoint,
+                                                      timeout=self.timeout,
+                                                      **{kwarg: kwargs})
+
+            try:
+                data = response.json()
+            except ValueError as e:
+                msg = "Make sure endpoint URL '%s' is correct." % self.endpoint
+                raise CloudStackException(
+                    "HTTP {0} response from CloudStack".format(
+                        response.status_code), response, "%s. " % str(e) + msg
+                    )
+
+            [key] = data.keys()
+            data = data[key]
+            if response.status_code != 200:
+                raise CloudStackException(
+                    "HTTP {0} response from CloudStack".format(
+                        response.status_code), response, data)
+            if fetch_list:
+                try:
+                    [key] = [k for k in data.keys() if k != 'count']
+                except ValueError:
+                    done = True
+                else:
+                    final_data.extend(data[key])
+                    page += 1
+            else:
+                final_data = data
+                done = True
+        return final_data
 
     def _sign(self, data):
         """
@@ -157,7 +177,9 @@ class CloudStack(object):
         return base64.b64encode(digest).decode('utf-8').strip()
 
 
-def read_config(ini_group='cloudstack'):
+def read_config(ini_group=None):
+    if not ini_group:
+        ini_group = os.environ.get('CLOUDSTACK_REGION', 'cloudstack')
     # Try env vars first
     os.environ.setdefault('CLOUDSTACK_METHOD', 'get')
     os.environ.setdefault('CLOUDSTACK_TIMEOUT', '10')
@@ -192,42 +214,41 @@ def read_config(ini_group='cloudstack'):
 
 
 def main():
-    usage = "Usage: {0} <command> [option1=value1 " \
-            "[option2=value2] ...] [--async] [--post] " \
-            "[--region=<region>]".format(sys.argv[0])
+    parser = argparse.ArgumentParser(description='Cloustack client.')
+    parser.add_argument('--region', metavar='REGION',
+                        help='Cloudstack region in ~/.cloudstack.ini',
+                        default=os.environ.get('CLOUDSTACK_REGION',
+                                               'cloudstack'))
+    parser.add_argument('--post', action='store_true', default=False,
+                        help='use POST instead of GET')
+    parser.add_argument('--async', action='store_true', default=False,
+                        help='do not wait for async result')
+    parser.add_argument('command', metavar="COMMAND",
+                        help='Cloudstack API command to execute')
 
-    if len(sys.argv) == 1:
-        raise SystemExit(usage)
+    def parse_option(x):
+        if '=' not in x:
+            raise ValueError("{!r} is not a correctly formatted "
+                             "option".format(x))
+        return x.split('=', 1)
 
-    command = sys.argv[1]
+    parser.add_argument('arguments', metavar="OPTION=VALUE",
+                        nargs='*', type=parse_option,
+                        help='Cloudstack API argument')
+
+    options = parser.parse_args()
+    command = options.command
     kwargs = defaultdict(set)
-    flags = set()
-    args = dict()
-    for option in sys.argv[2:]:
-        if option.startswith('--'):
-            option = option.strip('-')
-            if '=' in option:
-                key, value = option.split('=', 1)
-                if not value:
-                    raise SystemExit(usage)
-                args[key] = value
-            else:
-                flags.add(option)
-            continue
-        if '=' not in option:
-            raise SystemExit(usage)
-
-        key, value = option.split('=', 1)
+    for arg in options.arguments:
+        key, value = arg
         kwargs[key].add(value.strip(" \"'"))
 
-    region = args.get('region', 'cloudstack')
-
     try:
-        config = read_config(ini_group=region)
+        config = read_config(ini_group=options.region)
     except NoSectionError:
-        raise SystemExit("Error: region '%s' not in config" % region)
+        raise SystemExit("Error: region '%s' not in config" % options.region)
 
-    if 'post' in flags:
+    if options.post:
         config['method'] = 'post'
     cs = CloudStack(**config)
     try:
@@ -236,12 +257,12 @@ def main():
         response = e.args[2]
         sys.stderr.write("Cloudstack error:\n")
 
-    if 'Async' not in command and 'jobid' in response and 'async' not in flags:
+    if 'Async' not in command and 'jobid' in response and not options.async:
         sys.stderr.write("Polling result... ^C to abort\n")
         while True:
             try:
                 res = cs.queryAsyncJobResult(**response)
-                if res['jobprocstatus'] == 0:
+                if res['jobstatus'] != 0:
                     response = res
                     break
                 time.sleep(3)
